#include "framecpp/Common/Description.hh"
#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version6/FrameSpec.hh"
#include "framecpp/Version6/FrSE.hh"
#include "framecpp/Version6/FrSH.hh"
#include "framecpp/Version6/FrTOC.hh"

#include "framecpp/Version6/STRING.hh"

using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;

namespace FrameCPP
{
  namespace Version_6
  {
    //===================================================================
    //===================================================================
    FrTOCProcData::
    FrTOCProcData( )
    {
    }

    FrTOCProcData::
    FrTOCProcData( Common::IStream& Stream, INT_4U FrameCount )
    {
      nproc_type	nproc;
      Stream >> nproc;
      if ( nproc && ( nproc != FrTOC::NO_DATA_AVAILABLE ) )
      {
	//---------------------------------------------------------------
	// Read in the information
	//---------------------------------------------------------------
	key_container_type::size_type	s( nproc );
	std::vector< position_type >	positions( s * FrameCount );

	m_keys.resize( s );

	Stream >> m_keys
	       >> positions
	  ;
	//---------------------------------------------------------------
	// Move into structure.
	//---------------------------------------------------------------
	std::vector< position_type >::const_iterator
	  cur_position = positions.begin( );

	for ( key_container_type::const_iterator
		cur = m_keys.begin( ),
		last = m_keys.end( );
	      cur != last;
	      ++cur,
		cur_position += FrameCount )
	{
	  proc_info_type& i = m_info[ *cur ];
	  i.assign( cur_position, cur_position + FrameCount );
	}
      }
    }

    FrTOCProcData::MapProc_type::const_iterator FrTOCProcData::
    GetProc( const std::string& Channel ) const
    {
      return GetProc( ).find( Channel );
    }

    FrTOCProcData::MapProc_type::const_iterator FrTOCProcData::
    GetProc( INT_4U Channel ) const
    {
      if ( Channel >= GetProc( ).size( ) )
      {
	return GetProc( ).end( );
      }
      return GetProc( ).find( m_keys[ Channel ] );
    }

    void FrTOCProcData::
    QueryProc( const Common::TOCInfo& Info,
	       INT_4U FrameOffset,
	       INT_8U Position )
    {
      STRING	name;

      Info.TOCQuery( Common::TOCInfo::IC_NAME, Common::TOCInfo::DT_STRING_2, &name,
		     Common::TOCInfo::IC_EOQ );

      proc_info_type& i( m_info[ name ] );
      i.resize( FrameOffset + 1 );
      i[ FrameOffset ] = Position;
    }

    void FrTOCProcData::
    write( Common::OStream& Stream ) const
    {
      //-----------------------------------------------------------------
      // Flatten data so it is streamable
      //-----------------------------------------------------------------
      std::vector< name_type >::size_type	s( m_info.size( ) );
      if ( s )
      {
	std::vector< name_type >::size_type
	  fc( m_info.begin( )->second.size( ) );


	std::vector< name_type >	names( s );
	std::vector< position_type >	positions( s * fc );
	//---------------------------------------------------------------
	// Copy data for streaming
	//---------------------------------------------------------------
	std::vector< name_type >::iterator
	  cur_name = names.begin( );
	std::vector< position_type >::iterator
	  cur_position = positions.begin( );

	for ( MapProc_type::const_iterator
		cur = m_info.begin( ),
		last = m_info.end( );
	      cur != last;
	      ++cur, ++cur_name,
		cur_position += fc )
	{
	  *cur_name = cur->first;
	  std::copy( cur->second.begin( ),
		     cur->second.end( ),
		     cur_position );
	}
	//---------------------------------------------------------------
	// Stream out
	//---------------------------------------------------------------
	Stream << nproc_type( s )
	       << names
	       << positions
	  ;

      }
      else
      {
	Stream << nproc_type( FrTOC::NO_DATA_AVAILABLE );
      }
    }

  } // namespace - Version_6
} // namespace - FrameCPP
