#include "framecpp/Common/Description.hh"
#include "framecpp/Common/TOCInfo.hh"

#include "framecpp/Version8/FrameSpec.hh"
#include "framecpp/Version8/FrSE.hh"
#include "framecpp/Version8/FrSH.hh"
#include "framecpp/Version8/FrTOC.hh"

#include "framecpp/Version8/STRING.hh"

using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;
using FrameCPP::Common::TOCInfo;

namespace FrameCPP
{
  namespace Version_8
  {
    //===================================================================
    //===================================================================
    FrTOCProcData::
    FrTOCProcData( )
    {
    }

    FrTOCProcData::
    FrTOCProcData( Common::IStream& Stream, INT_4U FrameCount )
    {
      nproc_type	nproc;
      Stream >> nproc;
      if ( nproc && ( nproc != FrTOC::NO_DATA_AVAILABLE ) )
      {
	//---------------------------------------------------------------
	// Read in the information
	//---------------------------------------------------------------
	key_container_type::size_type	s( nproc );
	std::vector< position_type >	positions( s * FrameCount );

	m_keys.resize( s );

	Stream >> m_keys
	       >> positions
	  ;
	//---------------------------------------------------------------
	// Move into structure.
	//---------------------------------------------------------------
	std::vector< position_type >::const_iterator
	  cur_position = positions.begin( );

	for ( key_container_type::const_iterator
		cur = m_keys.begin( ),
		last = m_keys.end( );
	      cur != last;
	      ++cur,
		cur_position += FrameCount )
	{
	  proc_info_type& i = m_info[ *cur ];
	  i.assign( cur_position, cur_position + FrameCount );
	}
      }
    }

    FrTOCProcData::MapProc_type::const_iterator FrTOCProcData::
    GetProc( const std::string& Channel ) const
    {
      return GetProc( ).find( Channel );
    }

    FrTOCProcData::MapProc_type::const_iterator FrTOCProcData::
    GetProc( INT_4U Channel ) const
    {
      if ( Channel >= GetProc( ).size( ) )
      {
	return GetProc( ).end( );
      }
      return GetProc( ).find( m_keys[ Channel ] );
    }

    void FrTOCProcData::
    QueryProc( const Common::TOCInfo& Info,
	       INT_4U FrameOffset,
	       INT_8U Position )
    {
      STRING	name;

      Info.TOCQuery( TOCInfo::IC_NAME, TOCInfo::DT_STRING_2, &name,
		     TOCInfo::IC_EOQ );

      proc_info_type& i( m_info[ name ] );
      i.resize( FrameOffset + 1 );
      i[ FrameOffset ] = Position;
    }

    //-------------------------------------------------------------------
    /// This method allows for iterting over each element of information
    /// and allows the caller to gather information about each element.
    //-------------------------------------------------------------------
    void FrTOCProcData::
    forEach( Common::FrTOC::query_info_type Info,
	     Common::FrTOC::FunctionBase& Action ) const
    {
      switch( Info )
      {
      case Common::FrTOC::TOC_CHANNEL_NAMES:
	{
	  try
	  {
	    Common::FrTOC::FunctionString&
	      action( dynamic_cast< Common::FrTOC::FunctionString& >
		      ( Action ) );

	    for ( MapProc_type::const_iterator
		    cur = m_info.begin( ),
		    last = m_info.end( );
		  cur != last;
		  ++cur )
	    {
	      action( cur->first );
	    }
	  }
	  catch( ... )
	  {
	    // Does not understand Action
	  }
	}
	break;
      default:
	//---------------------------------------------------------------
	// ignore all other requests
	//---------------------------------------------------------------
	break;
      }
    }

    void FrTOCProcData::
    write( Common::OStream& Stream ) const
    {
      //-----------------------------------------------------------------
      // Flatten data so it is streamable
      //-----------------------------------------------------------------
      std::vector< name_type >::size_type	s( m_info.size( ) );
      if ( s )
      {
	std::vector< name_type >::size_type
	  fc( m_info.begin( )->second.size( ) );


	std::vector< name_type >	names( s );
	std::vector< position_type >	positions( s * fc );
	//---------------------------------------------------------------
	// Copy data for streaming
	//---------------------------------------------------------------
	std::vector< name_type >::iterator
	  cur_name = names.begin( );
	std::vector< position_type >::iterator
	  cur_position = positions.begin( );

	for ( MapProc_type::const_iterator
		cur = m_info.begin( ),
		last = m_info.end( );
	      cur != last;
	      ++cur, ++cur_name,
		cur_position += fc )
	{
	  *cur_name = cur->first;
	  std::copy( cur->second.begin( ),
		     cur->second.end( ),
		     cur_position );
	}
	//---------------------------------------------------------------
	// Stream out
	//---------------------------------------------------------------
	Stream << nproc_type( s )
	       << names
	       << positions
	  ;

      }
      else
      {
	Stream << nproc_type( FrTOC::NO_DATA_AVAILABLE );
      }
    }

    const FrTOCProcData& FrTOCProcData::
    operator=( const Previous::FrTOCProcData& Source )
    {
      m_keys = Source.GetKeys( );
      m_info = Source.GetProc( );

      return *this;
    }

  } // namespace - Version_8
} // namespace - FrameCPP
