#ifndef FrameCPP__VERSION_8__Specs_HH
#define FrameCPP__VERSION_8__Specs_HH

#include "ldastoolsal/types.hh"

#ifndef SWIGIMPORTED
#include "framecpp/Common/DetectorNames.hh"
#endif /* SWIGIMPORTED */
#include "framecpp/Common/FrameSpec.hh"

#if defined(__cplusplus) && !defined(SWIG)
namespace FrameCPP
{
  namespace Common {
    class OFrameStreamGen;
  }

  namespace Version_7 {}

  //---------------------------------------------------------------------
  /// \brief Implementation of the version 8 specification
  ///
  /// The version 8 frame specification is still under developement.
  /// With this being the case, this namespace should be considered as
  /// Alpha software and no production frames should be generated using
  /// this namespace.
  //---------------------------------------------------------------------
  namespace Version_8
  {
    namespace Previous = Version_7;

    typedef INT_4U checksum_type;
    typedef Common::FrameSpec::class_type class_type;

    template< typename ObjectType >
    class ObjectWithChecksum
      : public Common::FrameSpec::ObjectWithChecksum< ObjectType, checksum_type>
    {
    private:
      typedef Common::FrameSpec::ObjectWithChecksum< ObjectType, checksum_type> object_base_type;
    public:
      ObjectWithChecksum( )
      {
      }

      //-----------------------------------------------------------------
      /// \brief Constructor
      //-----------------------------------------------------------------
      ObjectWithChecksum( const Common::Description* Description )
	: object_base_type( Description )
      {
      }

      //-----------------------------------------------------------------
      /// \brief Constructor
      //-----------------------------------------------------------------
      ObjectWithChecksum( class_type Class,
			  const Common::Description* Description )
	: object_base_type( Class, Description )
      {
      }

      //-----------------------------------------------------------------
      /// \brief copy constructor
      //-----------------------------------------------------------------
      ObjectWithChecksum( const ObjectWithChecksum& Source )
	: object_base_type( Source )
      {
      }

      //-----------------------------------------------------------------
      /// \brief Constructor - Stream
      //-----------------------------------------------------------------
      ObjectWithChecksum( Common::IStream& Source )
	: object_base_type( Source )
      {
      }

      //-----------------------------------------------------------------
      /// \brief Destructor
      //-----------------------------------------------------------------
      ~ObjectWithChecksum( )
      {
      }

    };

    const CHAR_U DATA_FORMAT_VERSION = 8 ;
    const CHAR_U LIBRARY_MINOR_VERSION = 1;

    bool init_frame_spec( );

#ifndef SWIGIMPORTED
    extern const Common::DetectorNames&	DetectorNameTable;
#endif /* SWIGIMPORTED */

  } // namespace - Version_8
} // namespace - FrameCPP
#endif /* defined(__cplusplus) && !defined(SWIG) */
#endif /* FrameCPP__VERSION_8__Specs_HH */
